@@ -1,16 +1,29 @@
 package swat;
-our $VERSION = v0.1.14;
+
+our $VERSION = 'v0.1.17';
+
+use base 'Exporter'; 
+
+our @EXPORT = qw{version};
+
+sub version {
+    print $VERSION, "\n"
+}
+
+
 1;
 
 package main;
 use strict;
 use Test::More;
-our $HTTP_RESPONSE;
-our ($curl_cmd, $content_file, $url, $path, $http_meth, $debug, $ignore_http_err, $try_num, $debug_bytes );
-our ($a, $b);
 
+our $HTTP_RESPONSE;
+our ($curl_cmd, $content_file);
+our ($url, $path, $http_meth); 
+our ($debug, $ignore_http_err, $try_num, $debug_bytes);
 $| = 1;
 
+
 sub execute_with_retry {
 
     my $cmd = shift;
@@ -95,17 +108,19 @@ sub header {
 
 sub generate_asserts {
 
-
     my $filepath_or_array_ref = shift;
     my $write_header = shift;
 
     header() if $write_header;
 
-    my @ents = ();
+    my @ents;
+    my @ents_ok;
+    my $ent_type;
 
     if ( ref($filepath_or_array_ref) eq 'ARRAY') {
         @ents = @$filepath_or_array_ref
     }else{
+        return unless $filepath_or_array_ref;
         open my $fh, $filepath_or_array_ref or die $!;
         while (my $l = <$fh>){
             push @ents, $l
@@ -114,50 +129,132 @@ sub generate_asserts {
     }
 
 
-    my $comment;
 
-    for my $l (@ents){
+    ENTRY: for my $l (@ents){
 
         chomp $l;
+        diag $l if $ENV{'swat_debug'};
         
+        next ENTRY unless $l =~ /\S/; # skip blank lines
 
-        next unless $l =~ /\S/; # skip blank lines
         if ($l=~ /^\s*#(.*)/) { # skip comments
-            $comment = $1;
-            s/^\s+//, s/\s+$// for $comment;
-            next;
+            next ENTRY;
         }
 
-        if ($l=~/^\s*code:\s+(.*)/){
-            undef $comment;
+        if ($l=~/^\s*code:\s*(.*)/){
+            die "unterminated entity found: $ents_ok[-1]" if defined($ent_type);
             my $code = $1;
-            eval $code;
-        }elsif($l=~/^\s*generator:\s+(.*)/){
-            undef $comment;
+            if ($code=~s/\\\s*$//){
+                 push @ents_ok, $code;
+                 $ent_type = 'code';
+                 next ENTRY; # this is multiline, hold this until last line \ found
+            }else{
+                undef $ent_type;
+                handle_code($code);
+            }
+        }elsif($l=~/^\s*generator:\s*(.*)/){
+            die "unterminated entity found: $ents_ok[-1]" if defined($ent_type);
             my $code = $1;
-            my $arr_ref = eval $code;
-            generate_asserts($arr_ref,0);
+            if ($code=~s/\\\s*$//){
+                 push @ents_ok, $code;
+                 $ent_type = 'generator';
+                 next ENTRY; # this is multiline, hold this until last line \ found
+            }else{
+                undef $ent_type;
+                handle_generator($code);
+            }
             
-        }elsif($l=~/^\s*regexp:\s+(.*)/){
+        }elsif($l=~/^\s*regexp:\s*(.*)/){
+            die "unterminated entity found: $ents_ok[-1]" if defined($ent_type);
             my $re=$1;
-            my $message = $comment ? "[$comment] $http_meth $path returns data matching $re" : "$http_meth $path returns data matching $re";
-            check_line($re, 'regexp', $message);
-            undef $comment;
-        }else{
+            undef $ent_type;
+            handle_regexp($re);
+        }elsif(defined($ent_type)){
+            if ($l=~s/\\\s*$//) {
+                push @ents_ok, $l;
+                next ENTRY; # this is multiline, hold this until last line \ found
+             }else {
+
+                no strict 'refs';
+                my $name = "handle_"; $name.=$ent_type;
+                push @ents_ok, $l;
+                &$name(\@ents_ok);
+
+                undef $ent_type;
+                @ents_ok = ();
+    
+            }
+       }else{
             s{#.*}[], s{\s+$}[], s{^\s+}[] for $l;
-            my $message = $comment ? "[$comment] $http_meth $path returns $l" : "$http_meth $path returns $l";
-            check_line($l, 'default', $message);
-            undef $comment;
+            undef $ent_type;
+            handle_plain($l);
         }
     }
 
+    die "unterminated entity found: $ents_ok[-1]" if defined($ent_type);
 
 }
 
+sub handle_code {
+
+    my $code = shift;
+
+    unless (ref $code){
+        eval $code;
+        die "code entry eval perl error, code:$code , error: $@" if $@;
+        diag "handle_code OK. $code" if $ENV{'swat_debug'};
+    } else {
+        my $code_to_eval = join "\n", @$code;
+        eval $code_to_eval;
+        die "code entry eval error, code:$code_to_eval , error: $@" if $@;
+        diag "handle_code OK. multiline. $code_to_eval" if $ENV{'swat_debug'};
+    }
+    
+}
+
+sub handle_generator {
+
+    my $code = shift;
+
+    unless (ref $code){
+        my $arr_ref = eval $code;
+        die "generator entry eval error, code:$code , error: $@" if $@;
+        generate_asserts($arr_ref,0);
+        diag "handle_code OK. $code" if $ENV{'swat_debug'};
+    } else {
+        my $code_to_eval = join "\n", @$code;
+        my $arr_ref = eval $code_to_eval;
+        generate_asserts($arr_ref,0);
+        die "code entry eval error, code:$code_to_eval , error: $@" if $@;
+        diag "handle_generator OK. multiline. $code_to_eval" if $ENV{'swat_debug'};
+    }
+    
+}
+
+sub handle_regexp {
+
+    my $re = shift;
+    my $message = "$http_meth $path returns data matching $re";
+    check_line($re, 'regexp', $message);
+    diag "handle_regexp OK. $re" if $ENV{'swat_debug'};
+    
+}
+
+sub handle_plain {
+
+    my $l = shift;
+    my $message = "$http_meth $path returns $l";
+    check_line($l, 'default', $message);
+    diag "handle_plain OK. $l" if $ENV{'swat_debug'};   
+}
+
 
 
 1;
 
+=head1 ABSTRACT
+
+SWAT is Simple Web Application Test ( Tool )
 
 =head1 SYNOPSIS
 
@@ -210,7 +307,7 @@ has minimal dependency tree  and probably will run out of the box on most linux
 
 =item *
 
-has a simple and yet powerful DSL allow you to both run simple tests ( 200 OK ) or complicated ones ( using curl api and perl one-liners calls )
+has a simple and yet powerful DSL allow you to both run simple tests ( 200 OK ) or complicated ones ( using curl api and perl functions calls )
 
 =item *
 
@@ -230,20 +327,22 @@ and yes ... swat is fun :)
 =head2 Install swat
 
 
-=head3 developer release
-
-    sudo cpanm --mirror-only --mirror https://stratopan.com/melezhik/swat-release/master swat
-
 =head3 stable release
 
     sudo cpan install swat
 
+=head3 developer release
+
+    # developer release might be untested and unstable
+    sudo cpanm --mirror-only --mirror https://stratopan.com/melezhik/swat-release/master swat
+
+
 Once swat is installed you have B<swat> command line tool to run swat tests, but before do this you need to create them.
 
 
 =head2 Create tests
 
-    mkdir  my-app/ # create a project root directory to hold tests
+    mkdir  my-app/ # create a project root directory to contain tests
 
     # define http URIs application should response to
 
@@ -265,14 +364,13 @@ Once swat is installed you have B<swat> command line tool to run swat tests, but
 
 =head1 DSL
 
-Swat DSL consists of 2 parts. Routes and check patterns.
+Swat DSL consists of 2 parts. Routes and Swat Data.
 
 =head2 Routes
 
-Routes are http resources a tested web application should has.
+Routes are http resources a tested web application should have.
 
-Swat utilize file system I<representing> all existed routes as sub directories paths in the project root directory.
-Let we have a following project layout:
+Swat utilize file system to get know about routes. Let we have a following project layout:
 
     example/my-app/
     example/my-app/hello/
@@ -283,23 +381,52 @@ When you give swat a run
 
     swat example/my-app 127.0.0.1
 
-It will find all the directories holding get.txt files and "create" routes:
+It will find all the I<directories with get.txt|post.txt files inside> and "create" routes:
 
     GET hello/
     GET hello/world
 
-Then check patterns come into play.
+When you are done with routes you need to set swat data.
 
 
-=head2 Check patterns
+=head2 Swat data
 
-As you can see from tutorial above check patterns are  just text files describing B<what> is expected to return when route requested. Check patterns file parsed by swat line by line and take an action depending on entity found. There are 3 types of entities may be found in check patterns file:
+Swat data is DSL to describe/generate validation checks you apply to content returned from web application.
+Swat data is stored in swat data files, named get.txt or post.txt. 
+
+
+
+The process of validation looks like:
 
 =over
 
 =item *
 
-Expected Values
+Swat recursively find files named B<get.txt> or B<post.txt> in the project root directory to get swat data.
+
+=item *
+
+Swat parse swat data file and I<execute> entries found. At the end of this process swat creates a I<final check list> with 
+L</"Check Expressions">.
+
+=item *
+
+For every route swat makes http requests to web application and store content into text file 
+
+=item *
+
+Every line of text file is validated by every item in a I<final check list>
+
+
+=back 
+
+I<Objects> found in test data file are called I<swat entries>. There are I<3 basic type> of swat entries:
+
+=over
+
+=item *
+
+Check Expressions
 
 
 =item *
@@ -309,24 +436,33 @@ Comments
 
 =item *
 
-Perl one-liners code
+Perl Expressions and Generators
 
 
 =back
 
 
-=head3 Expected values
+=head3 Check Expressions
 
-This is most usable entity that one may define at check patterns files. I<It's just a string should be returned> when swat request a given URI. Here are examples:
+This is most usable type of entries you  may define at swat data file. I<It's just a string should be returned> when swat request a given URI. Here are examples:
 
     200 OK
     Hello World
     <head><title>Hello World</title></head>
 
 
+Using regexps
+
+Regexps are check expresions with the usage of <perl regular expressions> instead of plain strings checks.
+Everything started with C<regexp:> marker would be treated as perl regular expression.
+
+    # this is example of regexp check
+    regexp: App Version Number: (\d+\.\d+\.\d+)
+
+
 =head3 Comments
 
-Comments are lines started with '#' symbol, they are for humans not for swat which ignore comments when parse check pattern file. Here are examples.
+Comments entries are lines started with C<#> symbol, swat will ignore comments when parse swat data file. Here are examples.
 
     # this http status is expected
     200 OK
@@ -334,32 +470,115 @@ Comments are lines started with '#' symbol, they are for humans not for swat whi
     <head><title>Hello World</title></head> # and it should be proper html code
 
 
-=head3 Perl one-liners code
+=head3 Perl Expressions
 
-Everything started with C<code:> would be treated by swat as perl code to execute.
+Perl expressions are just a pieces of perl code to I<get evaled> by swat when parsing test data files.
+
+Everything started with C<code:> marker would be treated by swat as perl code to execute.
 There are a I<lot of possibilities>! Please follow L<Test::More|search.cpan.org/perldoc/Test::More> documentation to get more info about useful function you may call here.
 
     code: skip('next test is skipped',1) # skip next check forever
     HELLO WORLD
 
 
-=head3 Using regexp
+    code: skip('next test is skipped',1) unless $ENV{'debug'} == 1  # confitionally skip this check
+    HELLO SWAT
 
-Regexps are subtypes of expected values, with the only adjustment that you may use I<perl regular expressions> instead of plain strings checks.
-Everything started with C<regexp:> would be treated as regular expression.
 
-    # this is example of regexp check
-    regexp: App Version Number: (\d+\.\d+\.\d+)
+=head1 Generators
 
+Swat entries generators is the way to I<create new swat entries on the fly>. Technically specaking it's just a perl code which should return an array reference:
+Generators are very close to perl expressions ( generators code is alos get evaled ) with maijor difference:
+
+Value returned from generator's code should be  array reference. The array is passed back to swat parser so it can create new swat entries from it. 
+
+Generators entries start with C<:generator> marker. Here is example:
+
+    # Place this in swat pattern file
+    generator: [ qw{ foo bar baz } ]
+
+This generator will generate 3 swat entries:
+
+    foo
+    bar
+    baz
+
+
+
+As you can guess an array returned by generator should contain I<perl strings> representing swat entries, here is another example:
+with generator producing still 3 swat entites 'foo', 'bar', 'baz' :
+
+
+    # Place this in swat pattern file
+    generator: my %d = { 'foo' => 'foo value', 'bar' => 'bar value' }; [ map  { ( "# $_", "$data{$_}" )  } keys %d  ] 
+
+
+This generator will generate 3 swat entities:
+
+    # foo
+    foo value
+    # bar
+    bar value
+
+
+There is no limit for you! Use any code you want with only requiment - it should return array reference. 
+What about to validate web application content with sqlite database entries?
+
+    # Place this in swat pattern file
+    generator:                                                          \
+    
+    use DBI;                                                            \
+    my $dbh = DBI->connect("dbi:SQLite:dbname=t/data/test.db","","");   \
+    my $sth = $dbh->prepare("SELECT name from users");                  \
+    $sth->execute();                                                    \
+    my $results = $sth->fetchall_arrayref;                              \
+    
+    [ map { $_->[0] } @${results} ]
+
+See examples/swat-generators-sqlite3 for working example
+
+
+=head1 Multiline expressions
+
+Sometimes code looks more readable when you split it on separate chunks. When swat parser meets  C<\> symbols it postpone entry execution and
+add next line to buffer. This is repeated till no C<\> found on next. Finally swat execute I<"accumulated"> swat entity.
+
+Here are some exmaples:
+
+    # Place this in swat pattern file
+    generator:                  \
+    my %d = {                   \
+        'foo' => 'foo value',   \
+        'bar' => 'bar value',   \
+        'baz' => 'baz value'    \
+    };                          \
+    [                                               \
+        map  { ( "# $_", "$data{$_}" )  } keys %d   \
+    ]                                               \
+
+    # Place this in swat pattern file
+    generator: [            \
+            map {           \
+            uc($_)          \
+        } qw( foo bar baz ) \
+    ]
+
+    code:                                                       \
+    if $ENV{'debug'} == 1  { # confitionally skip this check    \
+        skip('next test is skipped',1)                          \ 
+    } 
+    HELLO SWAT
+
+Multiline expressions are only allowable for perl expressions and generators 
 
 =head1 Post requests
 
-When talking about swat I always say about Get http request, but swat may send a Post http request just name your check patterns file  as post.txt instead of get.txt
+Name swat data file as post.txt to make http POST requests.
 
     echo 200 OK >> my-app/hello/post.txt
     echo 200 OK >> my-app/hello/world/post.txt
 
-You may use curl_params setting ( follow L</"Swat settings"> section for details ) to define post data, there are some examples:
+You may use curl_params setting ( follow L</"Swat Settings"> section for details ) to define post data, there are some examples:
 
 =over
 
@@ -384,7 +603,14 @@ C<--data-binary> - Post data sending as is.
 
 =back
 
-=head1 Swat settings
+
+=head1 Generators and Perl Expressions Scope
+
+Swat uses I<perl string eval> when process generators and perl expressions code, be aware of this. 
+Follow L<http://perldoc.perl.org/functions/eval.html> to get more on this.
+
+
+=head1 Swat Settings
 
 Swat comes with settings defined in two contexts:
 
@@ -392,8 +618,7 @@ Swat comes with settings defined in two contexts:
 
 =item *
 
-environmental variables
-
+Environmental Variables
 
 =item *
 
@@ -403,7 +628,7 @@ swat.ini files
 =back
 
 
-=head2 Environmental variables
+=head2 Environmental Variables
 
 Defining a proper environment variables will provide swat settings.
 
@@ -449,10 +674,6 @@ C<curl_max_time> - follow curl documentation
 
 C<port>  - http port of tested host, default value is C<80>
 
-=item *
-
-C<noproxy>  - ignore http proxy when making http requests, default value is C<1>
-
 
 =back
 
@@ -583,6 +804,10 @@ C<-q -s> -  run tests in random and quite mode
 =back
 
 
+=head1 Debuggins
+
+set C<swat_debug> environmental variable to 1
+
 =head1 Examples
 
 ./examples directory contains examples of swat tests for different cases. Follow README.md files for details.
